@@ -48,6 +48,27 @@ extern gint  GdmUserMaxFile;
 extern gint  GdmRelaxPerms;
 extern gboolean GdmDebug;
 
+static void
+display_add_error (GdmDisplay *d)
+{
+	if (errno != 0)
+		gdm_error (_("%s: Could not write new authorization entry: %s"),
+			   "add_auth_entry", g_strerror (errno));
+	else
+		gdm_error (_("%s: Could not write new authorization entry.  "
+			     "Possibly out of diskspace"),
+			   "add_auth_entry");
+	if (d->console) {
+		char *s = g_strdup_printf
+			(_("GDM could not write a new authorization "
+			   "entry to disk.  Possibly out of diskspace.%s%s"),
+			 errno != 0 ? "  Error: " : "",
+			 errno != 0 ? g_strerror (errno) : "");
+		gdm_text_message_dialog (s);
+		g_free (s);
+	}
+}
+
 static gboolean
 add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 		unsigned short family, const char *addr, int addrlen)
@@ -58,8 +79,6 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 	if (!d)
 		return FALSE;
 
-	dispnum = g_strdup_printf ("%d", d->dispnum);
-
 	xa = malloc (sizeof (Xauth));
 
 	if (xa == NULL)
@@ -71,10 +90,16 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 		free (xa);
 		return FALSE;
 	}
+
+
 	memcpy (xa->address, addr, addrlen);
 	xa->address_length = addrlen;
+
+	dispnum = g_strdup_printf ("%d", d->dispnum);
 	xa->number = strdup (dispnum);
 	xa->number_length = strlen (dispnum);
+	g_free (dispnum);
+
 	xa->name = strdup ("MIT-MAGIC-COOKIE-1");
 	xa->name_length = strlen ("MIT-MAGIC-COOKIE-1");
 	xa->data = malloc (16);
@@ -88,14 +113,22 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 	memcpy (xa->data, d->bcookie, 16);
 	xa->data_length = 16;
 
-	XauWriteAuth (af, xa);
-	if (af2 != NULL)
-		XauWriteAuth (af2, xa);
+	errno = 0;
+	if ( ! XauWriteAuth (af, xa)) {
+		display_add_error (d);
+		return FALSE;
+	}
+
+	if (af2 != NULL) {
+		errno = 0;
+		if ( ! XauWriteAuth (af2, xa)) {
+			display_add_error (d);
+			return FALSE;
+		}
+	}
 
 	d->auths = g_slist_append (d->auths, xa);
 
-	g_free (dispnum);
-
 	return TRUE;
 }
 
@@ -321,6 +354,18 @@ gdm_auth_secure_display (GdmDisplay *d)
     return TRUE;
 }
 
+static gboolean
+try_open_append (const char *file)
+{
+	FILE *fp;
+	fp = fopen (file, "a+");
+	if (fp != NULL) {
+		fclose (fp);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
 
 /**
  * gdm_auth_user_add:
@@ -341,27 +386,42 @@ gdm_auth_user_add (GdmDisplay *d, uid_t user, const char *homedir)
     gint authfd;
     FILE *af;
     GSList *auths = NULL;
+    gboolean ret = TRUE;
+    gboolean automatic_tmp_dir = FALSE;
+    gboolean locked;
 
     if (!d)
 	return FALSE;
 
     gdm_debug ("gdm_auth_user_add: Adding cookie for %d", user);
 
+try_user_add_again:
+
+    locked = FALSE;
+
     /* Determine whether UserAuthDir is specified. Otherwise ~user is used */
-    if (*GdmUserAuthDir)
+    if ( ! ve_string_empty (GdmUserAuthDir))
 	authdir = GdmUserAuthDir;
     else
 	authdir = homedir;
 
     umask (077);
 
+    if (authdir == NULL)
+	    d->userauth = NULL;
+    else
+	    d->userauth = g_strconcat (authdir, "/", GdmUserAuthFile, NULL);
+
     /* Find out if the Xauthority file passes the paranoia check */
-    if (authdir == NULL ||
+    if (automatic_tmp_dir ||
+	authdir == NULL ||
 	! gdm_file_check ("gdm_auth_user_add", user, authdir, GdmUserAuthFile, 
-			  TRUE, GdmUserMaxFile, GdmRelaxPerms)) {
+			  TRUE, GdmUserMaxFile, GdmRelaxPerms) ||
+	! try_open_append (d->userauth)) {
 
 	/* No go. Let's create a fallback file in GdmUserAuthFB (/tmp) */
 	d->authfb = TRUE;
+	g_free (d->userauth);
 	d->userauth = g_strconcat (GdmUserAuthFB, "/.gdmXXXXXX", NULL);
 	authfd = g_mkstemp (d->userauth);
 
@@ -379,7 +439,6 @@ gdm_auth_user_add (GdmDisplay *d, uid_t user, const char *homedir)
     }
     else { /* User's Xauthority file is ok */
 	d->authfb = FALSE;
-	d->userauth = g_strconcat (authdir, "/", GdmUserAuthFile, NULL);
 
 	/* FIXME: Better implement my own locking. The libXau one is not kosher */
 	if (XauLockAuth (d->userauth, 3, 3, 0) != LOCK_SUCCESS) {
@@ -389,21 +448,30 @@ gdm_auth_user_add (GdmDisplay *d, uid_t user, const char *homedir)
 
 	    umask (022);
 
-	    return FALSE;
+	    automatic_tmp_dir = TRUE;
+	    goto try_user_add_again;
 	}
 
+	locked = TRUE;
+
 	af = fopen (d->userauth, "a+");
     }
 
     if (!af) {
 	/* Really no need to clean up here - this process is a goner anyway */
 	gdm_error (_("gdm_auth_user_add: Could not open cookie file %s"), d->userauth);
-	XauUnlockAuth (d->userauth);
+	if (locked)
+		XauUnlockAuth (d->userauth);
 	g_free (d->userauth);
 	d->userauth = NULL;
 
 	umask (022);
 
+	if ( ! d->authfb) {
+		automatic_tmp_dir = TRUE;
+		goto try_user_add_again;
+	}
+
 	return FALSE; 
     }
 
@@ -417,18 +485,36 @@ gdm_auth_user_add (GdmDisplay *d, uid_t user, const char *homedir)
     auths = d->auths;
 
     while (auths) {
-	XauWriteAuth (af, auths->data);
+	if ( ! XauWriteAuth (af, auths->data)) {
+		gdm_error (_("%s: Could not write cookie"),
+			   "gdm_auth_user_add");
+
+		if ( ! d->authfb) {
+			fclose (af);
+			if (locked)
+				XauUnlockAuth (d->userauth);
+			g_free (d->userauth);
+			d->userauth = NULL;
+			automatic_tmp_dir = TRUE;
+			goto try_user_add_again;
+		}
+
+		ret = FALSE;
+		break;
+	}
+
 	auths = auths->next;
     }
 
     fclose (af);
-    XauUnlockAuth (d->userauth);
+    if (locked)
+	    XauUnlockAuth (d->userauth);
 
     gdm_debug ("gdm_auth_user_add: Done");
 
     umask (022);
 
-    return TRUE;
+    return ret;
 }
 
 
@@ -482,14 +568,20 @@ gdm_auth_user_remove (GdmDisplay *d, uid_t user)
     g_free (authfile);
 
     /* Lock user's cookie jar and open it for writing */
-    if (XauLockAuth (d->userauth, 3, 3, 0) != LOCK_SUCCESS)
+    if (XauLockAuth (d->userauth, 3, 3, 0) != LOCK_SUCCESS) {
+	g_free (d->userauth);
+	d->userauth = NULL;
 	return;
+    }
 
     af = fopen (d->userauth, "a+");
 
     if (!af) {
 	XauUnlockAuth (d->userauth);
 
+	g_free (d->userauth);
+	d->userauth = NULL;
+
 	return;
     }
 
@@ -517,7 +609,7 @@ static void
 gdm_auth_purge (GdmDisplay *d, FILE *af)
 {
     Xauth *xa;
-    GSList *keep = NULL;
+    GSList *keep = NULL, *li;
 
     if (!d || !af)
 	return;
@@ -530,7 +622,7 @@ gdm_auth_purge (GdmDisplay *d, FILE *af)
      * temporary file issues. Then remove any instance of this display
      * in the cookie jar... */
 
-    while ( (xa = XauReadAuth (af)) ) {
+    while ( (xa = XauReadAuth (af)) != NULL ) {
 	gboolean match = FALSE;
 	GSList *alist = d->auths;
 
@@ -553,17 +645,14 @@ gdm_auth_purge (GdmDisplay *d, FILE *af)
     /* Rewind the file */
     af = freopen (d->userauth, "w", af);
 
-    if (!af) {
-	XauUnlockAuth (d->userauth);
-
-	return;
-    }
-
     /* Write out remaining entries */
-    while (keep) {
-	XauWriteAuth (af, keep->data);
-	XauDisposeAuth (keep->data);
-	keep = keep->next;
+    for (li = keep; li != NULL; li = li->next) {
+	    /* FIXME: is this correct, if we can't open
+	     * this is quite crap isn't it ... */
+	    if (af != NULL)
+		    XauWriteAuth (af, li->data);
+	    XauDisposeAuth (li->data);
+	    li->data = NULL;
     }
 
     g_slist_free (keep);
