@@ -30,27 +30,6 @@
 
 #include "gdm.h"
 
-/* If path starts with a "trusted" directory, don't sanity check things */
-static gboolean
-is_in_trusted_pic_dir (const char *path)
-{
-	char *globalpix;
-
-	/* our own pixmap dir is trusted */
-	if (strncmp (path, EXPANDED_PIXMAPDIR, sizeof (EXPANDED_PIXMAPDIR)) == 0)
-		return TRUE;
-
-	/* gnome's pixmap dir is trusted */
-	globalpix = gnome_unconditional_pixmap_file ("");
-	if (strncmp (path, globalpix, strlen (globalpix)) == 0) {
-		g_free (globalpix);
-		return TRUE;
-	}
-	g_free (globalpix);
-
-	return FALSE;
-}
-
 int
 main (int argc, char *argv[])
 {
@@ -131,21 +110,6 @@ main (int argc, char *argv[])
 						    _("No picture selected."));
 			gtk_dialog_run (GTK_DIALOG (d));
 			gtk_widget_destroy (d);
-		} else if (is_in_trusted_pic_dir (pixmap)) {
-			/* Picture is in trusted dir, no need to copy nor
-			 * check it */
-			char *homedir = g_get_home_dir ();
-			char *cfg_file = g_strconcat (homedir,
-						      "/.gnome/gdm",
-						      NULL);
-			gnome_config_set_string ("/gdm/face/picture",
-						 pixmap);
-			gnome_config_sync ();
-			/* ensure proper permissions */
-			chmod (cfg_file, 0600);
-			g_free (cfg_file);
-			g_free (homedir);
-			break;
 		} else if (s.st_size > max_size) {
 			GtkWidget *d;
 			d = gtk_message_dialog_new (NULL /* parent */,
@@ -163,11 +127,10 @@ main (int argc, char *argv[])
 		} else {
 			char buf[4096];
 			size_t size;
-			char *homedir = g_get_home_dir ();
-			char *photofile = g_strconcat (homedir,
+			char *photofile = g_strconcat (g_get_home_dir (),
 						       "/.gnome/photo",
 						       NULL);
-			char *cfg_file = g_strconcat (homedir,
+			char *cfg_file = g_strconcat (g_get_home_dir (),
 						      "/.gnome/gdm",
 						      NULL);
 			int fddest, fdsrc;
@@ -185,7 +148,6 @@ main (int argc, char *argv[])
 							    g_strerror (errno));
 				gtk_dialog_run (GTK_DIALOG (d));
 				gtk_widget_destroy (d);
-				g_free (homedir);
 				g_free (cfg_file);
 				g_free (photofile);
 				continue;
@@ -205,7 +167,6 @@ main (int argc, char *argv[])
 				gtk_dialog_run (GTK_DIALOG (d));
 				gtk_widget_destroy (d);
 				g_free (cfg_file);
-				g_free (homedir);
 				g_free (photofile);
 				close (fdsrc);
 				continue;
@@ -223,7 +184,6 @@ main (int argc, char *argv[])
 			/* ensure proper permissions */
 			chmod (cfg_file, 0600);
 			g_free (cfg_file);
-			g_free (homedir);
 			g_free (photofile);
 			break;
 		}
