@@ -24,6 +24,7 @@
 #include <gdk/gdkx.h>
 #include <X11/X.h>
 #include <X11/Xlib.h>
+#include <X11/Xatom.h>
 #ifdef HAVE_LIBXINERAMA
 #include <X11/extensions/Xinerama.h>
 #endif
@@ -51,6 +52,7 @@ static int no_focus_login = 0;
 static Display *wm_disp = NULL;
 static Window wm_root = None;
 static Window wm_login_window = None;
+static Window wm_focus_window = None;
 
 static Atom XA_WM_PROTOCOLS = 0;
 static Atom XA_WM_STATE = 0;
@@ -66,6 +68,24 @@ GdkRectangle gdm_wm_screen = {0,0,0,0};
 void 
 gdm_wm_screen_init (int cur_screen_num)
 {
+	if (g_getenv ("FAKE_XINERAMA_GDM") != NULL) {
+	/* for testing Xinerama support on non-xinerama setups */
+		gdm_wm_screen.x = 100;
+		gdm_wm_screen.y = 100;
+		gdm_wm_screen.width = gdk_screen_width () / 2 - 100;
+		gdm_wm_screen.height = gdk_screen_height () / 2 - 100;
+
+		gdm_wm_allscreens = g_new0 (GdkRectangle, 2);
+		gdm_wm_allscreens[0] = gdm_wm_screen;
+		gdm_wm_allscreens[1].x = gdk_screen_width () / 2;
+		gdm_wm_allscreens[1].y = gdk_screen_height () / 2;
+		gdm_wm_allscreens[1].width = gdk_screen_width () / 2;
+		gdm_wm_allscreens[1].height = gdk_screen_height () / 2;
+		gdm_wm_screens = 2;
+		return;
+	}
+
+	{
 #ifdef HAVE_LIBXINERAMA
 	gboolean have_xinerama = FALSE;
 
@@ -126,23 +146,7 @@ gdm_wm_screen_init (int cur_screen_num)
 		gdm_wm_allscreens[0] = gdm_wm_screen;
 		gdm_wm_screens = 1;
 	}
-#if 0
-	/* for testing Xinerama support on non-xinerama setups */
-	{
-		gdm_wm_screen.x = 100;
-		gdm_wm_screen.y = 100;
-		gdm_wm_screen.width = gdk_screen_width () / 2 - 100;
-		gdm_wm_screen.height = gdk_screen_height () / 2 - 100;
-
-		gdm_wm_allscreens = g_new0 (GdkRectangle, 2);
-		gdm_wm_allscreens[0] = gdm_wm_screen;
-		gdm_wm_allscreens[1].x = gdk_screen_width () / 2;
-		gdm_wm_allscreens[1].y = gdk_screen_height () / 2;
-		gdm_wm_allscreens[1].width = gdk_screen_width () / 2;
-		gdm_wm_allscreens[1].height = gdk_screen_height () / 2;
-		gdm_wm_screens = 2;
 	}
-#endif
 }
 
 void 
@@ -183,7 +187,7 @@ trap_push (void)
 	gdk_error_trap_push ();
 }
 
-static gboolean
+static int
 trap_pop (void)
 {
 	trap_depth --;
@@ -429,6 +433,8 @@ gdm_wm_focus_window (Window window)
 			RevertToPointerRoot,
 			CurrentTime);
 	trap_pop ();
+
+	wm_focus_window = window;
 }
 
 static void
@@ -536,7 +542,7 @@ has_deco (Window win)
 	trap_push ();
 
 	if (hints_atom == None)
-		hints_atom = XInternAtom (wm_disp, "_MOTIF_WM_HINTS", FALSE);
+		hints_atom = XInternAtom (wm_disp, "_MOTIF_WM_HINTS", False);
 
 	hints = NULL;
 
@@ -720,6 +726,9 @@ remove_window (Window w)
 {
 	GList *li = find_window_list (w, FALSE);
 
+	if (w == wm_focus_window)
+		wm_focus_window = None;
+
 	if (li != NULL) {
 		GdmWindow *gw = li->data;
 
@@ -1025,6 +1034,11 @@ static GSourceFuncs event_funcs = {
 	event_dispatch
 };
 
+#if 0
+/* ALEX: WTF is this? It's not used by anything, and I don't want
+ * the greeter to depend on libgnomeui and libbonoboui.
+ */
+
 /* HAAAAAAAAAAAAAACK */
 void
 bonobo_dock_item_grab_pointer (BonoboDockItem *item)
@@ -1046,6 +1060,7 @@ bonobo_dock_item_grab_pointer (BonoboDockItem *item)
 
   gdk_cursor_destroy (fleur);
 }
+#endif
 
 void
 gdm_wm_init (Window login_window)
@@ -1070,9 +1085,20 @@ gdm_wm_init (Window login_window)
 
 	XA_COMPOUND_TEXT = XInternAtom (wm_disp, "COMPOUND_TEXT", False);
 
-
 	wm_root = DefaultRootWindow (wm_disp);
 
+	/* set event mask for events on root window */
+	XGetWindowAttributes (wm_disp, wm_root, &attribs);
+	XSelectInput (wm_disp, wm_root,
+		      attribs.your_event_mask |
+		      SubstructureNotifyMask |
+		      SubstructureRedirectMask);
+
+	if (trap_pop () != 0)
+		return;
+
+	trap_push ();
+
 	add_all_current_windows ();
 
 	source = g_source_new (&event_funcs, sizeof (GSource));
@@ -1085,13 +1111,6 @@ gdm_wm_init (Window login_window)
 	g_source_set_can_recurse (source, FALSE);
 	g_source_attach (source, NULL);
 
-	/* set event mask for events on root window */
-	XGetWindowAttributes (wm_disp, wm_root, &attribs);
-	XSelectInput (wm_disp, wm_root,
-		      attribs.your_event_mask |
-		      SubstructureNotifyMask |
-		      SubstructureRedirectMask);
-
 	trap_pop ();
 }
 
@@ -1104,6 +1123,8 @@ gdm_wm_focus_new_windows (gboolean focus)
 void
 gdm_wm_no_login_focus_push (void)
 {
+	/* it makes not sense for this to be false then */
+	focus_new_windows = TRUE;
 	no_focus_login ++;
 }
 
@@ -1111,6 +1132,11 @@ void
 gdm_wm_no_login_focus_pop (void)
 {
 	no_focus_login --;
+
+	if (no_focus_login == 0 &&
+	    wm_focus_window == None &&
+	    wm_login_window != None)
+		gdm_wm_focus_window (wm_login_window);
 }
 
 void
@@ -1179,4 +1205,130 @@ gdm_wm_move_window_now (Window window, int x, int y)
 	trap_pop ();
 }
 
+void
+gdm_wm_save_wm_order (void)
+{
+	Window *children = NULL;
+	Window xparent, xroot;
+	guint size = 0;
+	int dlen = 0;
+	unsigned long *data;
+
+	gdk_flush ();
+	XSync (wm_disp, False);
+	trap_push ();
+
+	XGrabServer (wm_disp);
+
+	if (XQueryTree (wm_disp, 
+			wm_root,
+			&xroot,
+			&xparent,
+			&children,
+			&size)) {
+		int i;
+		Atom atom;
+		data = g_new0 (long, size);
+
+		for (i = 0; i < size; i++) {
+			GdmWindow *gw = find_window (children[i], TRUE);
+
+			/* Ignore unknowns and shadows */
+			if (gw == NULL ||
+			    gw->shadow == children[i])
+				continue;
+
+			if (gw->win == wm_login_window) {
+				/* Empty spot in the list signifies the
+				 * login window */
+				data [dlen++] = None;
+			} else {
+				data [dlen++] = gw->win;
+			}
+		}
+
+		atom = XInternAtom (wm_disp, "GDMWM_WINDOW_ORDER", False);
+
+		XChangeProperty (wm_disp, wm_root,
+				 atom,
+				 XA_CARDINAL,
+				 32,
+				 PropModeReplace,
+				 (unsigned char *)data,
+				 dlen);
+
+		if (children != NULL)
+			XFree (children);
+		g_free (data);
+	}
+
+	XUngrabServer (wm_disp);
+
+	trap_pop ();
+}
+
+static gboolean
+focus_win (gpointer data)
+{
+	Window focus = (Window)data;
+	focus_new_windows = TRUE;
+	gdm_wm_focus_window (focus);
+	return FALSE;
+}
+
+void
+gdm_wm_restore_wm_order (void)
+{
+	guint32 *data;
+	Window focus = None;
+	int size;
+	int i;
+	Atom atom;
+
+	gdk_flush ();
+	XSync (wm_disp, False);
+	trap_push ();
+
+	XGrabServer (wm_disp);
+
+	atom = XInternAtom (wm_disp, "GDMWM_WINDOW_ORDER", False);
+
+	data = get_typed_property_data (wm_disp, wm_root,
+					atom, XA_CARDINAL,
+					&size, 32);
+
+	if (data != NULL) {
+		for (i = 0; i < size/4; i++) {
+			GdmWindow *gw;
+			if (data[i] == None)
+				gw = find_window (wm_login_window, TRUE);
+			else
+				gw = find_window (data[i], TRUE);
+
+			if (gw != NULL) {
+				focus = gw->win;
+				if (gw->shadow != None)
+					XRaiseWindow (wm_disp, gw->shadow);
+				if (gw->deco != None)
+					XRaiseWindow (wm_disp, gw->deco);
+				else
+					XRaiseWindow (wm_disp, gw->win);
+			}
+		}
+
+		g_free (data);
+	}
+
+	XUngrabServer (wm_disp);
+
+	trap_pop ();
+
+	process_events ();
+
+	if (focus != None) {
+		/* let us hit the main loop first */ 
+		g_idle_add (focus_win, (gpointer)focus);
+	}
+}
+
 /* EOF */
